/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.description;

import java.net.URI;
import java.util.ArrayList;
import java.util.logging.Level;

import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.description.EMDConstants;
import com.ibm.j2ca.extension.emd.description.FaultBOUtil;
import com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIOutboundFunctionDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIOutboundServiceDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.internal.BusinessObjectDefinition;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataImportConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIMetadataConnectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.emd.MaxRecordSingleProperty;
import com.ibm.j2ca.sample.twineball.emd.discovery.TwineBallMetadataObject;
import com.ibm.j2ca.sample.twineball.emd.discovery.TwineBallMetadataSelection;
import com.ibm.j2ca.sample.twineball.emd.discovery.connection.TwineBallConfigurationProperties;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.FunctionDescription;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * define our outbound service.
 */
public class TwineBallOutboundServiceDescription extends WBIOutboundServiceDescriptionImpl {
    private static final String SETFUNCTIONDESCRIPTIONS = "setFunctionDescriptions()";
    public static final String CLASSNAME = "TwineBallOutboundServiceDescription";

    PropertyNameHelper helper;
    private LogUtils logUtils;

    public TwineBallOutboundServiceDescription( ){
    	super();
    }

    public TwineBallOutboundServiceDescription( PropertyNameHelper helper ){
    	super(helper);
    	this.helper = helper;
    	this.logUtils = helper.getLogUtils();
    }

    /*
     *
     * Set our function descriptions based on supported objects and functions.
     * @see com.ibm.j2ca.extension.emd.escription.WBIOutboundServiceDescriptionImpl#setFunctionDescriptions(commonj.connector.metadata.description.FunctionDescription[])
     */
    public void setFunctionDescriptions(MetadataSelection selection) throws MetadataException {
    	if(logUtils != null){
    		this.logUtils.traceMethodEntrance(CLASSNAME,SETFUNCTIONDESCRIPTIONS);
    	}
    	else{
    		WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME,SETFUNCTIONDESCRIPTIONS);
    	}
        ArrayList functionDescriptions = new ArrayList();
        MetadataImportConfiguration[] supportedObjects = selection.getSelection();
        PropertyGroup selectionProperties = ((WBIMetadataSelectionImpl) selection).getAppliedSelectionProperties();
        WBIMultiValuedPropertyImpl operationProperty = (WBIMultiValuedPropertyImpl) selectionProperties.getProperty("Operations");
        String[] supportedOperations = operationProperty.getValuesAsStrings();
        traceFiner("supportedOperations=" + supportedOperations);
        String location = TwineBallConfigurationProperties.getLocation(selectionProperties);

        for (int i = 0; i < supportedObjects.length; i++) {
            WBIMetadataImportConfigurationImpl importConfiguration = (WBIMetadataImportConfigurationImpl) supportedObjects[i];
            WBIOutboundFunctionDescriptionImpl outboundFunctionDescription = null;
            WBIInteractionSpec interactionSpec;
            TwineBallMetadataObject metadataObj = (TwineBallMetadataObject) importConfiguration.getMetadataObject();
            traceFiner("Object name is " + metadataObj.getBOName());
            for (int j = 0; j < supportedOperations.length; j++) {
                String operation = (String) supportedOperations[j];
                traceFiner("generating function for the " + operation + " operation on " + metadataObj.getBOName());
                if(helper != null){
                	 outboundFunctionDescription = new WBIOutboundFunctionDescriptionImpl(helper);
                }
                else{
                	outboundFunctionDescription = new WBIOutboundFunctionDescriptionImpl();
                }
                outboundFunctionDescription.setName(operation.toLowerCase() + metadataObj.getDisplayName());
                TwineBallDataDescription dataDescription = null;
                dataDescription = new TwineBallDataDescription(helper);
                dataDescription.setMetadataObject(metadataObj);
                dataDescription.setName(getNameSpace(), metadataObj.getDisplayName());
            	dataDescription.setRelativePath(location);
            	//dataDescription.setName(getNameSpace() + "/" + metadataObj.getBOName().toLowerCase() + "bg", metadataObj.getBOName() + "BG");

                //--CR87212 begins
            	WBISingleValuedPropertyImpl property = (WBISingleValuedPropertyImpl) selectionProperties.getProperty(EMDConstants.ARTIFACTS_SUPPORTED);
            	//WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.ALL, "Description", "setupMetadataObject", property.getValue()+"");

            	//Change made for making BG Optional
            	if (property.getValue().equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString()))
            		dataDescription.setTopLevel(true);
            	else
            		dataDescription.setTopLevel(false);

            	//This is just to force it to be generatedRecord, should be removed
            	//property.setValue(EMDConstants.GENERATED_RECORDS);
            	String namespace = getNameSpace();
            	//Change made for making BG Optional
            	if (property.getValue().equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString()))
            	{
                    dataDescription.populateSchemaDefinitions();

                    // Define XSDs for faults supported by this adapter.
                    try
                    {
                    	BusinessObjectDefinition bo = null;
                    	if(helper != null){
                    		bo = FaultBOUtil.createMissingDataBO(helper);
                    	}
                    	else{
                    	//	bo = FaultBOUtil.createMissingDataBO();
                    	}
                    	//BusinessObjectDefinition bo = FaultBOUtil.createMissingDataBO();
                    	URI uri = new URI("./" + FaultBOUtil.MISSING_DATA_NAME + EMDConstants.XSD);
                    	dataDescription.put(FaultBOUtil.FAULT_TARGET_NS, uri, bo.serialize());
                    	if(helper != null){
                    		bo = FaultBOUtil.createRecordNotFoundBO(helper);
                    	}
                    	else{
                    		bo = FaultBOUtil.createRecordNotFoundBO();
                    	}

                    	uri = new URI("./" + FaultBOUtil.RECORD_NOT_FOUND_NAME + EMDConstants.XSD);
                    	dataDescription.put(FaultBOUtil.FAULT_TARGET_NS, uri, bo.serialize());
                    }
                    catch (Exception e)
                    {
                    	throw new MetadataException("Unable to create fault BO definitions " + e.getMessage(), e); //$NON-NLS-1$
                    }

            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
        				+ metadataObj.getBOName().toLowerCase() + "bg" ), metadataObj //$NON-NLS-1$
        				.getBOName() + "BG"); //$NON-NLS-1$
            		dataDescription.setDataBindingGeneratorClassName("com.ibm.j2ca.sample.twineball.emd.runtime.TwineBallDataBindingGenerator");
            		dataDescription.setGenericDataBindingClassName(null);
                    outboundFunctionDescription.setInputDataDescription(dataDescription);

                    if (operation == WBIInteractionSpec.RETRIEVE_ALL_OP)
                    {
                        TwineBallDataDescription dd2 = new TwineBallDataDescription(helper);
                        dd2.setMetadataObject(metadataObj);
                        dd2.setName(getNameSpace(), metadataObj.getDisplayName());
                        dd2.setRelativePath(location);
                        dd2.setTopLevel(true);   // BG
                        dd2.setContainer(true);  // Container
                        dd2.populateSchemaDefinitions();
                        dd2.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" + metadataObj.getBOName().toLowerCase() + "container" ), metadataObj.getBOName() + "Container"); 
                        dd2.setDataBindingGeneratorClassName("com.ibm.j2ca.sample.twineball.emd.runtime.TwineBallDataBindingGenerator");
                        dd2.setGenericDataBindingClassName(null);
                        outboundFunctionDescription.setOutputDataDescription(dd2);
                    }
                    else
                    {
                        outboundFunctionDescription.setOutputDataDescription(dataDescription);
                    }
            	}
            	else if (property.getValue().equals(MetadataConfigurationType.GENERATED_RECORDS.toString()))
            	{
            	    dataDescription.populateSchemaDefinitions();
            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
            				+ metadataObj.getBOName().toLowerCase()), metadataObj //$NON-NLS-1$
            				.getBOName()); //$NON-NLS-1$
            		dataDescription.setDataBindingGeneratorClassName("com.ibm.j2ca.extension.dataexchange.bean.generator.RecordGenerator");

            		dataDescription.setGenericDataBindingClassName(null);

                    outboundFunctionDescription.setInputDataDescription(dataDescription);
                    outboundFunctionDescription.setOutputDataDescription(dataDescription);
            	} else {
            		//Generic Record Scenario
            	    dataDescription.populateSchemaDefinitions();
            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
            				+ metadataObj.getBOName().toLowerCase()), metadataObj //$NON-NLS-1$
            				.getBOName()); //$NON-NLS-1$
            		dataDescription.setGenericDataBindingClassName("com.ibm.j2ca.sample.twineball.TwineBallStructuredRecord");

                    outboundFunctionDescription.setInputDataDescription(dataDescription);
                    outboundFunctionDescription.setOutputDataDescription(dataDescription);
            	}
            	//End Change
                //--CR87212 ends
            	if(helper != null){
            		helper.getToolContext().getProgressMonitor().setNote("Business object definitions created");
            	}
            	else{
            		WBIMetadataConnectionImpl.getToolContext().getProgressMonitor().setNote("Business object definitions created");
            	}
                traceFiner("type of input object" + dataDescription.getName().getNamespaceURI() + "\\" + dataDescription.getName().getLocalPart());

                interactionSpec = new WBIInteractionSpec();
                if (operation.equals(WBIInteractionSpec.RETRIEVE_ALL_OP))
                {
                    MaxRecordSingleProperty maxCount = (MaxRecordSingleProperty) selectionProperties.getProperty(TwineBallMetadataSelection.MAXRECORDS);
                    if (maxCount.getValue() == null)
                        interactionSpec.setMaxRecords(((Integer) maxCount.getPropertyType().getDefaultValue()).intValue());
                    else
                        interactionSpec.setMaxRecords(((Integer) maxCount.getValue()).intValue());
                }
                interactionSpec.setFunctionName(operation);
                outboundFunctionDescription.setInteractionSpec(interactionSpec);
                outboundFunctionDescription.setImportConfiguration(importConfiguration);
                functionDescriptions.add(outboundFunctionDescription);
            }
        }
        FunctionDescription[] functionArray = new FunctionDescription[functionDescriptions.size()];
        functionDescriptions.toArray(functionArray);
        super.setFunctionDescriptions(functionArray);
        if(logUtils != null){
    		this.logUtils.traceMethodEntrance(CLASSNAME,SETFUNCTIONDESCRIPTIONS);
        }
    }



    /**
     * @param traceString
     */
    private void traceFiner(String traceString) {
    	if(logUtils != null){
    		this.logUtils.trace(Level.FINER, CLASSNAME, SETFUNCTIONDESCRIPTIONS,traceString);
    	}
        WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, SETFUNCTIONDESCRIPTIONS,traceString);
    }

}