/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery;

import java.rmi.RemoteException;
import java.util.*;
import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.*;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.internal.DisplayNameHelper;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.emd.description.TwineBallAttribute;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.MetadataObjectResponse;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This class allows us to retrieve the object information in each EIS node, and store it for later use.
 */
public class TwineBallMetadataObject extends WBIMetadataObjectImpl {
    private static final String OBJECTPROPERTIES = "ObjectProperties";
    TwineBallInterface twineBallConnection;
    private Object componentObject;
    private String prefix;
    private ArrayList childComponents;
    private LogUtils logUtils;
    PropertyNameHelper helper;

    /**
     * @return
     * @see commonj.connector.metadata.discovery.MetadataObject#isSelectableForImport()
     */
    public boolean isSelectableForImport() {
        return true;
    }

    /**
     * We set up the object's properties right in the constructor.
     * @param twineBallConnection
     * @param twineBallObjectName
     */

    TwineBallMetadataObject(TwineBallInterface twineBallConnection, String twineBallObjectName, PropertyNameHelper helper ) throws MetadataException{
    	super(helper);
    	this.helper = helper;
    	this.twineBallConnection = twineBallConnection;
        this.setBOName(twineBallObjectName);
        this.logUtils = helper.getLogUtils();
        try {
            logUtils.trace(Level.FINER,TwineBallMetadataObject.class.getName(),"Constructor","Getting columns for TwineBall Object: " + twineBallObjectName);
            LinkedHashMap attributes = new LinkedHashMap();
            Vector columnNames = twineBallConnection.getColumnNames(twineBallObjectName);
            setProperties(columnNames, false, attributes);
            Vector children = twineBallConnection.getChildNames(twineBallObjectName);
            setProperties(children, true, attributes);
        } catch (RemoteException e) {
            logUtils.trace(Level.SEVERE,TwineBallMetadataObject.class.getName(),"Constructor","Connection failed", e);
            throw new MetadataException(e.getMessage(),e);
        } catch (TwineBallException e) {
            logUtils.trace(Level.SEVERE,TwineBallMetadataObject.class.getName(), "Constructor", "Unable to retrieve attributes", e);
            throw new MetadataException(e.getMessage(),e);
        }
    }


    private void setProperties(Vector columnNames, boolean children, LinkedHashMap attributes) {
        Iterator propertyIterator = columnNames.iterator();
        while (propertyIterator.hasNext()) {
            String propertyName = (String) propertyIterator.next();
            attributes.put(propertyName, new TwineBallAttribute(children));
        }
        this.setAttributes(attributes);
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.MetadataObject#createFilteringProperties()
     */
    public PropertyGroup createFilteringProperties() {
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.MetadataObject#getObjectProperties()
     */
    public PropertyGroup getObjectProperties() {
        WBIPropertyGroupImpl propertyGroup = null;
        try {
            logUtils.trace(Level.FINER,TwineBallMetadataObject.class.getName(),"getChildComponents","Getting object properties for " + this.getDisplayName());
            if(helper != null){
	            propertyGroup = new WBIPropertyGroupImpl("Object properties", helper);
	            propertyGroup.setDisplayName(helper.getPropertyName(OBJECTPROPERTIES));
	            propertyGroup.setDescription(helper.getPropertyDescription(OBJECTPROPERTIES));
	            WBISingleValuedPropertyImpl objectName = new WBISingleValuedPropertyImpl("Object Name", String.class, helper);
	            objectName.setDisplayName(helper.getPropertyName("ObjectName"));
	            objectName.setDescription(helper.getPropertyDescription("ObjectName"));
	            objectName.setValue(this.getDisplayName());
            }
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propertyGroup;
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.MetadataObject#getChildren(commonj.connector.metadata.discovery.properties.PropertyGroup,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public MetadataObjectResponse getChildren(PropertyGroup filterParameters) throws MetadataException {
    	WBIMetadataObjectResponseImpl response = null;
    	if(helper != null){
    		response = new WBIMetadataObjectResponseImpl(helper);
        }
        ArrayList objects = getChildComponents();
        response.setObjects(objects);
        return response;
    }

    private ArrayList getChildComponents() throws MetadataException {
        //caching the child component list... I can do this easily because I'm
        // not supporting filter properties.   This will be called multiple times.
        if (childComponents == null) {
            logUtils.trace(Level.FINER,TwineBallMetadataObject.class.getName(),"getChildComponents","Getting child components");
            String currentObjectName = this.getBOName();
            Vector childObjects = null;
            try {
                childObjects = twineBallConnection.getChildNames(currentObjectName);
            } catch (RemoteException e) {
                logUtils.trace(Level.SEVERE,TwineBallMetadataObject.class.getName(),"getChildComponents","Connection failed", e);
                throw new MetadataException(e.getMessage(),e);
            } catch (TwineBallException e) {
                logUtils.trace(Level.SEVERE,TwineBallMetadataObject.class.getName(),"getChildComponents","Unable to get child components", e);
                throw new MetadataException(e.getMessage(),e);
            }
            TwineBallMetadataObjectListFactory twineballObjListFactory = new TwineBallMetadataObjectListFactory ();
            if(helper != null){
            	childComponents =twineballObjListFactory.getMetadataObjectsForObjectNames(childObjects, twineBallConnection, logUtils,helper);
            }
        }
        return childComponents;
    }
}