/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery.connection;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.internal.DisplayNameHelper;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This is our connection configuration for oubound.  We don't need much beyond the base properties.
 */
public class TwineBallOutboundConnectionConfiguration extends WBIOutboundConnectionConfigurationImpl {

	EMDUtil emdUtil;
	LogUtils logUtils;
	PropertyNameHelper helper;

    public TwineBallOutboundConnectionConfiguration(WBIOutboundConnectionTypeImpl connType,PropertyNameHelper helper) throws MetadataException {
        super(connType, helper);
        emdUtil = new EMDUtil();
        this.logUtils = helper.getLogUtils();
        this.helper = helper;
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#getUnifiedProperties()
     */
    public PropertyGroup createUnifiedProperties() {
        WBIPropertyGroupImpl propGroup = null;

        if (this.getOutboundConnectionType().isSupportedInMetadataService()) {
            try {
            	if(logUtils != null){
            		 propGroup = TwineBallConfigurationProperties.getTwineBallConfigurationProperties(helper);
            	}
            } catch (MetadataException e) {
                throw new RuntimeException(e);
            }
        } else //Create a property group for runtime
        {
            try {
            	if(logUtils != null){
	                propGroup = new WBIPropertyGroupImpl("Outbound Properties", helper);
	                propGroup.setDisplayName(helper.getPropertyName("OutboundProperties"));
	                propGroup.setDescription(helper.getPropertyDescription("OutboundProperties"));
	                propGroup.addProperty(createManagedConnectionFactoryProperties());
            	}
            } catch (MetadataException e) {
            	throw new RuntimeException(e);
            }
        }
        try {
            if (getAppliedProperties() != null){
            	if(emdUtil != null){
            		emdUtil.copyValues(getAppliedProperties(), propGroup,helper);
            	}
            }
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propGroup;
    }

    /**
     * @return
     * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createResourceAdapterProperties()
     */
    public PropertyGroup createResourceAdapterProperties() {
        TwineBallResourceAdapter ra = new TwineBallResourceAdapter();
        WBIPropertyGroupImpl adapterProp = null;
        try {
        	if(emdUtil != null){
        		adapterProp = ((WBIPropertyGroupImpl) emdUtil.getPropertyGroup(((WBIOutboundConnectionTypeImpl) this.getOutboundConnectionType()).getResourceAdapterJavaBean(), helper));
        		if (getAppliedProperties() != null && adapterProp != null)
        			emdUtil.copyValues(getAppliedProperties(), adapterProp, helper);
        	}
        } catch (MetadataException e) {
            throw new RuntimeException(e.getMessage());
        }
        return adapterProp;
    }

    public PropertyGroup createManagedConnectionFactoryProperties() {
        WBIPropertyGroupImpl connProp = null;
        try {
        	if(emdUtil != null){
	            connProp = ((WBIPropertyGroupImpl) emdUtil.getPropertyGroup(((WBIOutboundConnectionTypeImpl) this.getOutboundConnectionType()).getManagedConnectionFactoryJavaBean(),helper));
	            WBISingleValuedPropertyImpl url = new WBISingleValuedPropertyImpl("Url", String.class, helper);
	            url.setRequired(false);
	            url.setDisplayName(helper.getPropertyName("Url"));
	            url.setDescription(helper.getPropertyDescription("Url"));
	            connProp.addProperty(url);
	            WBISingleValuedPropertyImpl schema = new WBISingleValuedPropertyImpl("Schema", String.class, helper);
	            schema.setDisplayName(helper.getPropertyName("Schema"));
	            schema.setDescription(helper.getPropertyDescription("Schema"));
	            schema.setRequired(false);
	            connProp.addProperty(schema);
		    connProp.setExpert(true);
	            if (getAppliedProperties() != null)
	                emdUtil.copyValues(getAppliedProperties(), connProp, helper);
        	}
        } catch (MetadataException e) {
            throw new RuntimeException(e.getMessage());
        }
        return connProp;
    }
}